
// Calculate flux over one face given two neighbouring cells on each side

/* 
Assume scalars:  
SX
SY
SZ

SH
SL

SXH
SYH

SXL
SYL
SZL
SHL

rho_LC1
rho_LC2
rho_LC3
rho_LC4

U_LC1
U_LC2
U_LC3
U_LC4

rhoU_LC2
rhoU_LC3

p_LC1
p_LC2
p_LC3
p_LC4

rhoE_LC2
rhoE_LC3

c_LC2
c_LC3

are defined and scalars:

F1
F2
F3
F4
F5

exist. 
*/


{
  // COMPUTE LEFT- & RIGHT-BIASED RO,U,P AT CELL WALLS
  scalar ROL = 
    COEF1.value()*rho_LC1 + 
    COEF2.value()*rho_LC2 + 
    COEF3.value()*rho_LC3 + 
    COEF4.value()*rho_LC4;
  vector UL = 
    COEF1.value()*U_LC1 + 
    COEF2.value()*U_LC2 + 
    COEF3.value()*U_LC3 + 
    COEF4.value()*U_LC4;
  scalar PL = 
    COEF1.value()*p_LC1 + 
    COEF2.value()*p_LC2 + 
    COEF3.value()*p_LC3 + 
    COEF4.value()*p_LC4;
  
  scalar ROR = 
    COEF4.value()*rho_LC1 + 
    COEF3.value()*rho_LC2 + 
    COEF2.value()*rho_LC3 + 
    COEF1.value()*rho_LC4;
  vector UR = 
    COEF4.value()*U_LC1 + 
    COEF3.value()*U_LC2 + 
    COEF2.value()*U_LC3 + 
    COEF1.value()*U_LC4;
  scalar PR = 
    COEF4.value()*p_LC1 + 
    COEF3.value()*p_LC2 + 
    COEF2.value()*p_LC3 + 
    COEF1.value()*p_LC4;
  
  // COMPUTE AVERAGE RO,U,V,W,C AT CELL WALL
  scalar ROA = 0.5*(rho_LC2 + rho_LC3);
  vector UA  = 0.5*(  U_LC2 +   U_LC3);
  scalar CA  = 0.5*(  c_LC2 +   c_LC3); 
  
  // COMPUTE EIGENVALUES
  
  scalar EV1 = SX*UA.x() + SY*UA.y() + SZ*UA.z();
  scalar EV2 = EV1;
  scalar EV3 = EV1;
  scalar EV4 = EV1 + CA*SL;
  scalar EV5 = EV1 - CA*SL;
  
  // COMPUTE UPWIND-BIASED CHARACT. VARIABLES
  
  scalar R1;
  scalar R2;
  scalar R3;
  scalar R4;
  scalar R5;
  
  if (EV1 < 0.) 
    {R1 = ROR - PR/sqr(CA);}
  else 
    {R1 = ROL - PL/sqr(CA);}
  
  if (EV2 < 0.) 
    {R2 = -SYH*UR.x() + SXH*UR.y();}
  else 
    {R2 = -SYH*UL.x() + SXH*UL.y();}
  
  if (EV3 < 0.) 
    {R3 = -SZL*(SXH*UR.x()+SYH*UR.y()) + SHL*UR.z();}
  else 
    {R3 = -SZL*(SXH*UL.x()+SYH*UL.y()) + SHL*UL.z();}
  
  if (EV4 < 0.) 
    {R4 = .5*ROA/CA*(SXL*UR.x()+SYL*UR.y()+SZL*UR.z())+.5*PR/sqr(CA);}
  else
    {R4 = .5*ROA/CA*(SXL*UL.x()+SYL*UL.y()+SZL*UL.z())+.5*PL/sqr(CA);}
  
  if (EV5 < 0.)
    {R5 =-.5*ROA/CA*(SXL*UR.x()+SYL*UR.y()+SZL*UR.z())+.5*PR/sqr(CA);}
  else
    {R5 =-.5*ROA/CA*(SXL*UL.x()+SYL*UL.y()+SZL*UL.z())+.5*PL/sqr(CA);}
  
  // TRANSFORM BACK TO PRIMITIVE VARIABLES
  scalar ROM = R1 + R4 + R5;
  scalar UM  = -SYH*R2 - SXH*SZL*R3 + CA/ROA*SXL*(R4-R5);
  scalar VM  =  SXH*R2 - SYH*SZL*R3 + CA/ROA*SYL*(R4-R5);
  scalar WM  =  SHL*R3              + CA/ROA*SZL*(R4-R5);
  scalar PM  = sqr(CA) * (R4+R5);
  scalar ETM = PM/(gamma.value()-1.) + .5*ROM*(sqr(UM)+sqr(VM)+sqr(WM));
  
  // COMPUTE LOCAL DAMPING COEFFICIENT
  
  scalar D2PA = mag(p_LC1-2.*p_LC2+p_LC3) / 
    mag(p_LC1+2.*p_LC2+p_LC3);
  scalar D2PB = mag(p_LC2-2.*p_LC3+p_LC4) / 
    mag(p_LC2+2.*p_LC3+p_LC4);
  scalar D2P  = max(D2PA,D2PB);
  
  scalar D2RA = mag(rho_LC1-2.*rho_LC2+rho_LC3) / 
    mag(rho_LC1+2.*rho_LC2+rho_LC3);
  scalar D2RB = mag(rho_LC2-2.*rho_LC3+rho_LC4) / 
    mag(rho_LC2+2.*rho_LC3+rho_LC4);
  scalar D2R  = max(D2RA,D2RB);
  
  scalar CDMP = FCNTP.value() * D2P * (mag(EV1)+CA*SL);
  scalar CDMC = FCNTC.value() * D2R * mag(EV1);
  
  scalar CDAM = max(CDMP,CDMC);
  
  //scalar D2QA = mag(Q(LC1)-2.*Q(LC2)+Q(LC3)) / 
  //              mag(Q(LC1)+2.*Q(LC2)+Q(LC3));
  //scalar D2QB = mag(Q(LC2)-2.*Q(LC3)+Q(LC4)) / 
  //              mag(Q(LC2)+2.*Q(LC3)+Q(LC4));
  //scalar D2Q = max(D2QA,D2QB);
  //
  //scalar CQDF = FCNTQ * D2Q * ROM * mag(EV1);
  //
  //scalar D2EA = mag(EPS(LC1)-2.*EPS(LC2)+EPS(LC3)) /  
  //              mag(EPS(LC1)+2.*EPS(LC2)+EPS(LC3));
  //scalar D2EB = mag(EPS(LC2)-2.*EPS(LC3)+EPS(LC4)) /  
  //              mag(EPS(LC2)+2.*EPS(LC3)+EPS(LC4));
  //scalar D2E = max(D2EA,D2EB);
  //scalar CEDF = FCNTE * D2E * ROM * mag(EV1);
  
  
  // COMPUTE FLUXES
  
  F1 = ROM*UM*SX + ROM*VM*SY + ROM*WM*SZ - 
    CDAM*(rho_LC3-rho_LC2);
  
  F2 = (ROM*sqr(UM)+PM)*SX + (ROM*UM*VM)*SY + (ROM*UM*WM)*SZ - 
    CDAM*(rhoU_LC3.x()-rhoU_LC2.x());
  
  F3 = (ROM*VM*UM)*SX + (ROM*sqr(VM)+PM)*SY + (ROM*VM*WM)*SZ - 
    CDAM*(rhoU_LC3.y()-rhoU_LC2.y());
  
  F4 = (ROM*WM*UM)*SX + (ROM*WM*VM)*SY + (ROM*sqr(WM)+PM)*SZ - 
    CDAM*(rhoU_LC3.z()-rhoU_LC2.z());
  
  F5 = (ETM+PM)*UM*SX + (ETM+PM)*VM*SY + (ETM+PM)*WM*SZ - 
    CDAM*(rhoE_LC3-rhoE_LC2);
  
  //scalar F6 = ROM*UM*QM*SX + ROM*VM*QM*SY + ROM*WM*QM*SZ - 
  //CDAM*(RQ(LC3)-RQ(LC2)) - 
  //CQDF*(Q(LC3)-Q(LC2));
  //
  //scalar F7 = ROM*UM*EPSM*SX + ROM*VM*EPSM*SY + ROM*WM*EPSM*SZ - 
  //CDAM*(REPS(LC3)-REPS(LC2)) - CEDF*(EPS(LC3)-EPS(LC2));
  
}
  

